@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -37,22 +35,16 @@
 #include <unixlib/local.h>
 #endif
 
-
-#if HAVE_TIME_H
-#include <time.h>
-#endif
 #if HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_SIGNAL_H
+
 #include <signal.h>
-#endif
-#if HAVE_SETLOCALE
 #include <locale.h>
-#endif
+
 #if HAVE_DLFCN_H
 #include <dlfcn.h>
 #endif
@@ -99,14 +91,23 @@
 
 #include "php_http_parser.h"
 #include "php_cli_server.h"
+#include "php_cli_server_arginfo.h"
 #include "mime_type_map.h"
 
 #include "php_cli_process_title.h"
+#include "php_cli_process_title_arginfo.h"
 
 #define OUTPUT_NOT_CHECKED -1
 #define OUTPUT_IS_TTY 1
 #define OUTPUT_NOT_TTY 0
 
+#if HAVE_FORK
+# include <sys/wait.h>
+static pid_t	 php_cli_server_master;
+static pid_t	*php_cli_server_workers;
+static zend_long php_cli_server_workers_max;
+#endif
+
 typedef struct php_cli_server_poller {
 	fd_set rfds, wfds;
 	struct {
@@ -208,7 +209,13 @@ static php_cli_server_http_response_status_code_pair template_map[] = {
 	{ 501, "<h1>%s</h1><p>Request method not supported.</p>" }
 };
 
-#if HAVE_UNISTD_H
+#define PHP_CLI_SERVER_LOG_PROCESS 1
+#define PHP_CLI_SERVER_LOG_ERROR   2
+#define PHP_CLI_SERVER_LOG_MESSAGE 3
+
+static int php_cli_server_log_level = 3;
+
+#if HAVE_UNISTD_H || defined(PHP_WIN32)
 static int php_cli_output_is_tty = OUTPUT_NOT_CHECKED;
 #endif
 
@@ -217,10 +224,10 @@ static const char php_cli_server_request_error_unexpected_eof[] = "Unexpected EO
 static size_t php_cli_server_client_send_through(php_cli_server_client *client, const char *str, size_t str_len);
 static php_cli_server_chunk *php_cli_server_chunk_heap_new_self_contained(size_t len);
 static void php_cli_server_buffer_append(php_cli_server_buffer *buffer, php_cli_server_chunk *chunk);
-static void php_cli_server_logf(const char *format, ...);
+static void php_cli_server_logf(int type, const char *format, ...);
 static void php_cli_server_log_response(php_cli_server_client *client, int status, const char *message);
 
-ZEND_DECLARE_MODULE_GLOBALS(cli_server);
+ZEND_DECLARE_MODULE_GLOBALS(cli_server)
 
 /* {{{ static char php_cli_server_css[]
  * copied from ext/standard/info.c
@@ -256,7 +263,7 @@ int php_cli_server_get_system_time(char *buf) {
 
 	gettimeofday(&tv, NULL);
 
-	/* TODO: should be checked for NULL tm/return vaue */
+	/* TODO: should be checked for NULL tm/return value */
 	php_localtime_r(&tv.tv_sec, &tm);
 	php_asctime_r(&tm, buf);
 	return 0;
@@ -268,7 +275,7 @@ static void char_ptr_dtor_p(zval *zv) /* {{{ */
 	pefree(Z_PTR_P(zv), 1);
 } /* }}} */
 
-static char *get_last_error() /* {{{ */
+static char *get_last_error(void) /* {{{ */
 {
 	return pestrdup(strerror(errno), 1);
 } /* }}} */
@@ -341,23 +348,37 @@ static void append_http_status_line(smart_str *buffer, int protocol_version, int
 	smart_str_appendl_ex(buffer, "\r\n", 2, persistent);
 } /* }}} */
 
-static void append_essential_headers(smart_str* buffer, php_cli_server_client *client, int persistent) /* {{{ */
+static void append_essential_headers(smart_str* buffer, php_cli_server_client *client, int persistent, sapi_headers_struct *sapi_headers) /* {{{ */
 {
 	char *val;
 	struct timeval tv = {0};
+	bool append_date_header = true;
 
-	if (NULL != (val = zend_hash_str_find_ptr(&client->request.headers, "host", sizeof("host")-1))) {
-		smart_str_appendl_ex(buffer, "Host", sizeof("Host") - 1, persistent);
-		smart_str_appendl_ex(buffer, ": ", sizeof(": ") - 1, persistent);
-		smart_str_appends_ex(buffer, val, persistent);
-		smart_str_appendl_ex(buffer, "\r\n", 2, persistent);
+	if (sapi_headers != NULL) {
+		zend_llist_position pos;
+		sapi_header_struct *h = (sapi_header_struct*)zend_llist_get_first_ex(&sapi_headers->headers, &pos);
+		while (h) {
+			if (h->header_len > strlen("Date:")) {
+				if (strncasecmp(h->header, "Date:", strlen("Date:")) == 0) {
+					append_date_header = false;
+					break;
+				}
+			}
+			h = (sapi_header_struct*)zend_llist_get_next_ex(&sapi_headers->headers, &pos);
+		}
 	}
 
-	if (!gettimeofday(&tv, NULL)) {
-		zend_string *dt = php_format_date("r", 1, tv.tv_sec, 1);
-		smart_str_appendl_ex(buffer, "Date: ", 6, persistent);
+	if (NULL != (val = zend_hash_str_find_ptr(&client->request.headers, "host", sizeof("host")-1))) {
+		smart_str_appends_ex(buffer, "Host: ", persistent);
+		smart_str_appends_ex(buffer, val, persistent);
+		smart_str_appends_ex(buffer, "\r\n", persistent);
+	}
+
+	if (append_date_header && !gettimeofday(&tv, NULL)) {
+		zend_string *dt = php_format_date("D, d M Y H:i:s", sizeof("D, d M Y H:i:s") - 1, tv.tv_sec, 0);
+		smart_str_appends_ex(buffer, "Date: ", persistent);
 		smart_str_appends_ex(buffer, dt->val, persistent);
-		smart_str_appendl_ex(buffer, "\r\n", 2, persistent);
+		smart_str_appends_ex(buffer, " GMT\r\n", persistent);
 		zend_string_release_ex(dt, 0);
 	}
 
@@ -366,7 +387,13 @@ static void append_essential_headers(smart_str* buffer, php_cli_server_client *c
 
 static const char *get_mime_type(const php_cli_server *server, const char *ext, size_t ext_len) /* {{{ */
 {
-	return (const char*)zend_hash_str_find_ptr(&server->extension_mime_types, ext, ext_len);
+	char *ret;
+	ALLOCA_FLAG(use_heap)
+	char *ext_lower = do_alloca(ext_len + 1, use_heap);
+	zend_str_tolower_copy(ext_lower, ext, ext_len);
+	ret = zend_hash_str_find_ptr(&server->extension_mime_types, ext_lower, ext_len);
+	free_alloca(ext_lower, use_heap);
+	return (const char*)ret;
 } /* }}} */
 
 PHP_FUNCTION(apache_request_headers) /* {{{ */
@@ -378,7 +405,7 @@ PHP_FUNCTION(apache_request_headers) /* {{{ */
 	zval tmp;
 
 	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	client = SG(server_context);
@@ -424,7 +451,7 @@ static void add_response_header(sapi_header_struct *h, zval *return_value) /* {{
 PHP_FUNCTION(apache_response_headers) /* {{{ */
 {
 	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	array_init(return_value);
@@ -432,8 +459,7 @@ PHP_FUNCTION(apache_response_headers) /* {{{ */
 }
 /* }}} */
 
-/* {{{ cli_server module
- */
+/* {{{ cli_server module */
 
 static void cli_server_init_globals(zend_cli_server_globals *cg)
 {
@@ -476,24 +502,18 @@ zend_module_entry cli_server_module_entry = {
 };
 /* }}} */
 
-ZEND_BEGIN_ARG_INFO(arginfo_no_args, 0)
-ZEND_END_ARG_INFO()
-
 const zend_function_entry server_additional_functions[] = {
-	PHP_FE(cli_set_process_title,        arginfo_cli_set_process_title)
-	PHP_FE(cli_get_process_title,        arginfo_cli_get_process_title)
-	PHP_FE(apache_request_headers, arginfo_no_args)
-	PHP_FE(apache_response_headers, arginfo_no_args)
-	PHP_FALIAS(getallheaders, apache_request_headers, arginfo_no_args)
+	PHP_FE(cli_set_process_title,	arginfo_cli_set_process_title)
+	PHP_FE(cli_get_process_title,	arginfo_cli_get_process_title)
+	PHP_FE(apache_request_headers,	arginfo_apache_request_headers)
+	PHP_FE(apache_response_headers,	arginfo_apache_response_headers)
+	PHP_FALIAS(getallheaders, apache_request_headers, arginfo_getallheaders)
 	PHP_FE_END
 };
 
 static int sapi_cli_server_startup(sapi_module_struct *sapi_module) /* {{{ */
 {
-	if (php_module_startup(sapi_module, &cli_server_module_entry, 1) == FAILURE) {
-		return FAILURE;
-	}
-	return SUCCESS;
+	return php_module_startup(sapi_module, &cli_server_module_entry, 1);
 } /* }}} */
 
 static size_t sapi_cli_server_ub_write(const char *str, size_t str_length) /* {{{ */
@@ -547,7 +567,7 @@ static int sapi_cli_server_send_headers(sapi_headers_struct *sapi_headers) /* {{
 		append_http_status_line(&buffer, client->request.protocol_version, SG(sapi_headers).http_response_code, 0);
 	}
 
-	append_essential_headers(&buffer, client, 0);
+	append_essential_headers(&buffer, client, 0, sapi_headers);
 
 	h = (sapi_header_struct*)zend_llist_get_first_ex(&sapi_headers->headers, &pos);
 	while (h) {
@@ -696,10 +716,14 @@ static void sapi_cli_server_register_variables(zval *track_vars_array) /* {{{ */
 	zend_hash_apply_with_arguments(&client->request.headers, (apply_func_args_t)sapi_cli_server_register_entry_cb, 1, track_vars_array);
 } /* }}} */
 
-static void sapi_cli_server_log_message(char *msg, int syslog_type_int) /* {{{ */
+static void sapi_cli_server_log_write(int type, const char *msg) /* {{{ */
 {
 	char buf[52];
 
+	if (php_cli_server_log_level < type) {
+		return;
+	}
+
 	if (php_cli_server_get_system_time(buf) != 0) {
 		memmove(buf, "unknown time, can't be fetched", sizeof("unknown time, can't be fetched"));
 	} else {
@@ -710,11 +734,23 @@ static void sapi_cli_server_log_message(char *msg, int syslog_type_int) /* {{{ *
 			memmove(buf, "unknown", sizeof("unknown"));
 		}
 	}
+#ifdef HAVE_FORK
+	if (php_cli_server_workers_max > 1) {
+		fprintf(stderr, "[%ld] [%s] %s\n", (long) getpid(), buf, msg);
+	} else {
+		fprintf(stderr, "[%s] %s\n", buf, msg);
+	}
+#else
 	fprintf(stderr, "[%s] %s\n", buf, msg);
+#endif
 } /* }}} */
 
-/* {{{ sapi_module_struct cli_server_sapi_module
- */
+static void sapi_cli_server_log_message(const char *msg, int syslog_type_int) /* {{{ */
+{
+	sapi_cli_server_log_write(PHP_CLI_SERVER_LOG_MESSAGE, msg);
+} /* }}} */
+
+/* {{{ sapi_module_struct cli_server_sapi_module */
 sapi_module_struct cli_server_sapi_module = {
 	"cli-server",							/* name */
 	"Built-in HTTP server",		/* pretty name */
@@ -845,14 +881,14 @@ static int php_cli_server_poller_iter_on_active(php_cli_server_poller *poller, v
 
 	for (fd=0 ; fd<=max_fd ; fd++)  {
 		if (PHP_SAFE_FD_ISSET(fd, &poller->active.rfds)) {
-                if (SUCCESS != callback(opaque, fd, POLLIN)) {
-                    retval = FAILURE;
-                }
+				if (SUCCESS != callback(opaque, fd, POLLIN)) {
+					retval = FAILURE;
+				}
 		}
 		if (PHP_SAFE_FD_ISSET(fd, &poller->active.wfds)) {
-                if (SUCCESS != callback(opaque, fd, POLLOUT)) {
-                    retval = FAILURE;
-                }
+				if (SUCCESS != callback(opaque, fd, POLLOUT)) {
+					retval = FAILURE;
+				}
 		}
 	}
 #endif
@@ -1065,9 +1101,11 @@ static int php_cli_server_content_sender_pull(php_cli_server_content_sender *sen
 	_nbytes_read = read(fd, chunk->data.heap.p, chunk->data.heap.len);
 #endif
 	if (_nbytes_read < 0) {
-		char *errstr = get_last_error();
-		php_cli_server_logf("%s", errstr);
-		pefree(errstr, 1);
+		if (php_cli_server_log_level >= PHP_CLI_SERVER_LOG_ERROR) {
+			char *errstr = get_last_error();
+			php_cli_server_logf(PHP_CLI_SERVER_LOG_ERROR, "%s", errstr);
+			pefree(errstr, 1);
+		}
 		php_cli_server_chunk_dtor(chunk);
 		pefree(chunk, 1);
 		return 1;
@@ -1079,39 +1117,41 @@ static int php_cli_server_content_sender_pull(php_cli_server_content_sender *sen
 } /* }}} */
 
 #if HAVE_UNISTD_H
-static int php_cli_is_output_tty() /* {{{ */
+static int php_cli_is_output_tty(void) /* {{{ */
 {
 	if (php_cli_output_is_tty == OUTPUT_NOT_CHECKED) {
 		php_cli_output_is_tty = isatty(STDOUT_FILENO);
 	}
 	return php_cli_output_is_tty;
 } /* }}} */
+#elif defined(PHP_WIN32)
+static int php_cli_is_output_tty() /* {{{ */
+{
+	if (php_cli_output_is_tty == OUTPUT_NOT_CHECKED) {
+		php_cli_output_is_tty = php_win32_console_fileno_is_console(STDOUT_FILENO) && php_win32_console_fileno_has_vt100(STDOUT_FILENO);
+	}
+	return php_cli_output_is_tty;
+} /* }}} */
 #endif
 
 static void php_cli_server_log_response(php_cli_server_client *client, int status, const char *message) /* {{{ */
 {
 	int color = 0, effective_status = status;
 	char *basic_buf, *message_buf = "", *error_buf = "";
-	zend_bool append_error_message = 0;
+	bool append_error_message = 0;
 
 	if (PG(last_error_message)) {
-		switch (PG(last_error_type)) {
-			case E_ERROR:
-			case E_CORE_ERROR:
-			case E_COMPILE_ERROR:
-			case E_USER_ERROR:
-			case E_PARSE:
-				if (status == 200) {
-					/* the status code isn't changed by a fatal error, so fake it */
-					effective_status = 500;
-				}
+		if (PG(last_error_type) & E_FATAL_ERRORS) {
+			if (status == 200) {
+				/* the status code isn't changed by a fatal error, so fake it */
+				effective_status = 500;
+			}
 
-				append_error_message = 1;
-				break;
+			append_error_message = 1;
 		}
 	}
 
-#if HAVE_UNISTD_H
+#if HAVE_UNISTD_H || defined(PHP_WIN32)
 	if (CLI_SERVER_G(color) && php_cli_is_output_tty() == OUTPUT_IS_TTY) {
 		if (effective_status >= 500) {
 			/* server error: red */
@@ -1127,7 +1167,7 @@ static void php_cli_server_log_response(php_cli_server_client *client, int statu
 #endif
 
 	/* basic */
-	spprintf(&basic_buf, 0, "%s [%d]: %s", client->addr_str, status, client->request.request_uri);
+	spprintf(&basic_buf, 0, "%s [%d]: %s %s", client->addr_str, status, php_http_method_str(client->request.request_method), client->request.request_uri);
 	if (!basic_buf) {
 		return;
 	}
@@ -1143,7 +1183,8 @@ static void php_cli_server_log_response(php_cli_server_client *client, int statu
 
 	/* error */
 	if (append_error_message) {
-		spprintf(&error_buf, 0, " - %s in %s on line %d", PG(last_error_message), PG(last_error_file), PG(last_error_lineno));
+		spprintf(&error_buf, 0, " - %s in %s on line %d",
+			ZSTR_VAL(PG(last_error_message)), ZSTR_VAL(PG(last_error_file)), PG(last_error_lineno));
 		if (!error_buf) {
 			efree(basic_buf);
 			if (message) {
@@ -1154,9 +1195,9 @@ static void php_cli_server_log_response(php_cli_server_client *client, int statu
 	}
 
 	if (color) {
-		php_cli_server_logf("\x1b[3%dm%s%s%s\x1b[0m", color, basic_buf, message_buf, error_buf);
+		php_cli_server_logf(PHP_CLI_SERVER_LOG_MESSAGE, "\x1b[3%dm%s%s%s\x1b[0m", color, basic_buf, message_buf, error_buf);
 	} else {
-		php_cli_server_logf("%s%s%s", basic_buf, message_buf, error_buf);
+		php_cli_server_logf(PHP_CLI_SERVER_LOG_MESSAGE, "%s%s%s", basic_buf, message_buf, error_buf);
 	}
 
 	efree(basic_buf);
@@ -1168,11 +1209,15 @@ static void php_cli_server_log_response(php_cli_server_client *client, int statu
 	}
 } /* }}} */
 
-static void php_cli_server_logf(const char *format, ...) /* {{{ */
+static void php_cli_server_logf(int type, const char *format, ...) /* {{{ */
 {
 	char *buf = NULL;
 	va_list ap;
 
+	if (php_cli_server_log_level < type) {
+		return;
+	}
+
 	va_start(ap, format);
 	vspprintf(&buf, 0, format, ap);
 	va_end(ap);
@@ -1181,9 +1226,7 @@ static void php_cli_server_logf(const char *format, ...) /* {{{ */
 		return;
 	}
 
-	if (sapi_module.log_message) {
-		sapi_module.log_message(buf, -1);
-	}
+	sapi_cli_server_log_write(type, buf);
 
 	efree(buf);
 } /* }}} */
@@ -1300,9 +1343,6 @@ out:
 
 static int php_cli_server_request_ctor(php_cli_server_request *req) /* {{{ */
 {
-#ifdef ZTS
-ZEND_TSRMLS_CACHE_UPDATE();
-#endif
 	req->protocol_version = 0;
 	req->request_uri = NULL;
 	req->request_uri_len = 0;
@@ -1315,7 +1355,9 @@ ZEND_TSRMLS_CACHE_UPDATE();
 	req->query_string = NULL;
 	req->query_string_len = 0;
 	zend_hash_init(&req->headers, 0, NULL, char_ptr_dtor_p, 1);
+	GC_MAKE_PERSISTENT_LOCAL(&req->headers);
 	zend_hash_init(&req->headers_original_case, 0, NULL, NULL, 1);
+	GC_MAKE_PERSISTENT_LOCAL(&req->headers_original_case);
 	req->content = NULL;
 	req->content_len = 0;
 	req->ext = NULL;
@@ -1458,6 +1500,7 @@ static void normalize_vpath(char **retval, size_t *retval_len, const char *vpath
 	char *p;
 
 	*retval = NULL;
+	*retval_len = 0;
 
 	decoded_vpath = pestrndup(vpath, vpath_len, persistent);
 	if (!decoded_vpath) {
@@ -1543,6 +1586,9 @@ static int php_cli_server_client_read_request_on_path(php_http_parser *parser, c
 	{
 		char *vpath;
 		size_t vpath_len;
+		if (UNEXPECTED(client->request.vpath != NULL)) {
+			return 1;
+		}
 		normalize_vpath(&vpath, &vpath_len, at, length, 1);
 		client->request.vpath = vpath;
 		client->request.vpath_len = vpath_len;
@@ -1553,17 +1599,34 @@ static int php_cli_server_client_read_request_on_path(php_http_parser *parser, c
 static int php_cli_server_client_read_request_on_query_string(php_http_parser *parser, const char *at, size_t length)
 {
 	php_cli_server_client *client = parser->data;
-	client->request.query_string = pestrndup(at, length, 1);
-	client->request.query_string_len = length;
+	if (EXPECTED(client->request.query_string == NULL)) {
+		client->request.query_string = pestrndup(at, length, 1);
+		client->request.query_string_len = length;
+	} else {
+		ZEND_ASSERT(length <= PHP_HTTP_MAX_HEADER_SIZE && PHP_HTTP_MAX_HEADER_SIZE - length >= client->request.query_string_len);
+		client->request.query_string = perealloc(client->request.query_string, client->request.query_string_len + length + 1, 1);
+		memcpy(client->request.query_string + client->request.query_string_len, at, length);
+		client->request.query_string_len += length;
+		client->request.query_string[client->request.query_string_len] = '\0';
+	}
 	return 0;
 }
 
 static int php_cli_server_client_read_request_on_url(php_http_parser *parser, const char *at, size_t length)
 {
 	php_cli_server_client *client = parser->data;
-	client->request.request_method = parser->method;
-	client->request.request_uri = pestrndup(at, length, 1);
-	client->request.request_uri_len = length;
+	if (EXPECTED(client->request.request_uri == NULL)) {
+		client->request.request_method = parser->method;
+		client->request.request_uri = pestrndup(at, length, 1);
+		client->request.request_uri_len = length;
+	} else {
+		ZEND_ASSERT(client->request.request_method == parser->method);
+		ZEND_ASSERT(length <= PHP_HTTP_MAX_HEADER_SIZE && PHP_HTTP_MAX_HEADER_SIZE - length >= client->request.query_string_len);
+		client->request.request_uri = perealloc(client->request.request_uri, client->request.request_uri_len + length + 1, 1);
+		memcpy(client->request.request_uri + client->request.request_uri_len, at, length);
+		client->request.request_uri_len += length;
+		client->request.request_uri[client->request.request_uri_len] = '\0';
+	}
 	return 0;
 }
 
@@ -1601,7 +1664,7 @@ static int php_cli_server_client_read_request_on_header_field(php_http_parser *p
 	switch (client->last_header_element) {
 	case HEADER_VALUE:
 		php_cli_server_client_save_header(client);
-		/* break missing intentionally */
+		ZEND_FALLTHROUGH;
 	case HEADER_NONE:
 		client->current_header_name = (char *)at;
 		client->current_header_name_len = length;
@@ -1666,7 +1729,7 @@ static int php_cli_server_client_read_request_on_headers_complete(php_http_parse
 		client->current_header_value = pemalloc(1, 1);
 		*client->current_header_value = '\0';
 		client->current_header_value_len = 0;
-		/* break missing intentionally */
+		ZEND_FALLTHROUGH;
 	case HEADER_VALUE:
 		php_cli_server_client_save_header(client);
 		break;
@@ -1737,20 +1800,30 @@ static int php_cli_server_client_read_request(php_cli_server_client *client, cha
 		if (err == SOCK_EAGAIN) {
 			return 0;
 		}
-		*errstr = php_socket_strerror(err, NULL, 0);
+
+		if (php_cli_server_log_level >= PHP_CLI_SERVER_LOG_ERROR) {
+			*errstr = php_socket_strerror(err, NULL, 0);
+		}
+
 		return -1;
 	} else if (nbytes_read == 0) {
-		*errstr = estrdup(php_cli_server_request_error_unexpected_eof);
+		if (php_cli_server_log_level >= PHP_CLI_SERVER_LOG_ERROR) {
+			*errstr = estrdup(php_cli_server_request_error_unexpected_eof);
+		}
+
 		return -1;
 	}
 	client->parser.data = client;
 	nbytes_consumed = php_http_parser_execute(&client->parser, &settings, buf, nbytes_read);
 	if (nbytes_consumed != (size_t)nbytes_read) {
-		if (buf[0] & 0x80 /* SSLv2 */ || buf[0] == 0x16 /* SSLv3/TLSv1 */) {
-			*errstr = estrdup("Unsupported SSL request");
-		} else {
-			*errstr = estrdup("Malformed HTTP request");
+		if (php_cli_server_log_level >= PHP_CLI_SERVER_LOG_ERROR) {
+			if ((buf[0] & 0x80) /* SSLv2 */ || buf[0] == 0x16 /* SSLv3/TLSv1 */) {
+				*errstr = estrdup("Unsupported SSL request");
+			} else {
+				*errstr = estrdup("Malformed HTTP request");
+			}
 		}
+
 		return -1;
 	}
 	if (client->current_header_name) {
@@ -1785,12 +1858,8 @@ static size_t php_cli_server_client_send_through(php_cli_server_client *client,
 				int nfds = php_pollfd_for(client->sock, POLLOUT, &tv);
 				if (nfds > 0) {
 					continue;
-				} else if (nfds < 0) {
-					/* error */
-					php_handle_aborted_connection();
-					return nbytes_left;
 				} else {
-					/* timeout */
+					/* error or timeout */
 					php_handle_aborted_connection();
 					return nbytes_left;
 				}
@@ -1874,9 +1943,8 @@ static void php_cli_server_client_dtor(php_cli_server_client *client) /* {{{ */
 
 static void php_cli_server_close_connection(php_cli_server *server, php_cli_server_client *client) /* {{{ */
 {
-#if PHP_DEBUG
-	php_cli_server_logf("%s Closing", client->addr_str);
-#endif
+	php_cli_server_logf(PHP_CLI_SERVER_LOG_MESSAGE, "%s Closing", client->addr_str);
+
 	zend_hash_index_del(&server->clients, client->sock);
 } /* }}} */
 
@@ -1891,7 +1959,7 @@ static int php_cli_server_send_error_page(php_cli_server *server, php_cli_server
 	php_cli_server_content_sender_ctor(&client->content_sender);
 	client->content_sender_initialized = 1;
 
-	escaped_request_uri = php_escape_html_entities_ex((unsigned char *)client->request.request_uri, client->request.request_uri_len, 0, ENT_QUOTES, NULL, 0);
+	escaped_request_uri = php_escape_html_entities_ex((const unsigned char *) client->request.request_uri, client->request.request_uri_len, 0, ENT_QUOTES, NULL, /* double_encode */ 0, /* quiet */ 0);
 
 	{
 		static const char prologue_template[] = "<!doctype html><html><head><title>%d %s</title>";
@@ -1944,7 +2012,7 @@ static int php_cli_server_send_error_page(php_cli_server *server, php_cli_server
 			/* out of memory */
 			goto fail;
 		}
-		append_essential_headers(&buffer, client, 1);
+		append_essential_headers(&buffer, client, 1, NULL);
 		smart_str_appends_ex(&buffer, "Content-Type: text/html; charset=UTF-8\r\n", 1);
 		smart_str_appends_ex(&buffer, "Content-Length: ", 1);
 		smart_str_append_unsigned_ex(&buffer, php_cli_server_buffer_size(&client->content_sender.buffer), 1);
@@ -1983,14 +2051,12 @@ static int php_cli_server_dispatch_script(php_cli_server *server, php_cli_server
 	}
 	{
 		zend_file_handle zfd;
-		zfd.type = ZEND_HANDLE_FILENAME;
-		zfd.filename = SG(request_info).path_translated;
-		zfd.handle.fp = NULL;
-		zfd.free_filename = 0;
-		zfd.opened_path = NULL;
+		zend_stream_init_filename(&zfd, SG(request_info).path_translated);
+		zfd.primary_script = 1;
 		zend_try {
 			php_execute_script(&zfd);
 		} zend_end_try();
+		zend_destroy_file_handle(&zfd);
 	}
 
 	php_cli_server_log_response(client, SG(sapi_headers).http_response_code, NULL);
@@ -2042,7 +2108,7 @@ static int php_cli_server_begin_send_static(php_cli_server *server, php_cli_serv
 			php_cli_server_log_response(client, 500, NULL);
 			return FAILURE;
 		}
-		append_essential_headers(&buffer, client, 1);
+		append_essential_headers(&buffer, client, 1, NULL);
 		if (mime_type) {
 			smart_str_appendl_ex(&buffer, "Content-Type: ", sizeof("Content-Type: ") - 1, 1);
 			smart_str_appends_ex(&buffer, mime_type, 1);
@@ -2077,8 +2143,6 @@ static int php_cli_server_request_startup(php_cli_server *server, php_cli_server
 	}
 	SG(sapi_headers).http_response_code = 200;
 	if (FAILURE == php_request_startup()) {
-		/* should never be happen */
-		destroy_request_info(&SG(request_info));
 		return FAILURE;
 	}
 	PG(during_request_startup) = 0;
@@ -2108,11 +2172,8 @@ static int php_cli_server_dispatch_router(php_cli_server *server, php_cli_server
 	old_cwd[0] = '\0';
 	php_ignore_value(VCWD_GETCWD(old_cwd, MAXPATHLEN - 1));
 
-	zfd.type = ZEND_HANDLE_FILENAME;
-	zfd.filename = server->router;
-	zfd.handle.fp = NULL;
-	zfd.free_filename = 0;
-	zfd.opened_path = NULL;
+	zend_stream_init_filename(&zfd, server->router);
+	zfd.primary_script = 1;
 
 	zend_try {
 		zval retval;
@@ -2123,11 +2184,11 @@ static int php_cli_server_dispatch_router(php_cli_server *server, php_cli_server
 				decline = Z_TYPE(retval) == IS_FALSE;
 				zval_ptr_dtor(&retval);
 			}
-		} else {
-			decline = 1;
 		}
 	} zend_end_try();
 
+	zend_destroy_file_handle(&zfd);
+
 	if (old_cwd[0] != '\0') {
 		php_ignore_value(VCWD_CHDIR(old_cwd));
 	}
@@ -2141,17 +2202,18 @@ static int php_cli_server_dispatch_router(php_cli_server *server, php_cli_server
 static int php_cli_server_dispatch(php_cli_server *server, php_cli_server_client *client) /* {{{ */
 {
 	int is_static_file  = 0;
+	const char *ext = client->request.ext;
 
 	SG(server_context) = client;
-	if (client->request.ext_len != 3 || memcmp(client->request.ext, "php", 3) || !client->request.path_translated) {
+	if (client->request.ext_len != 3
+	 || (ext[0] != 'p' && ext[0] != 'P') || (ext[1] != 'h' && ext[1] != 'H') || (ext[2] != 'p' && ext[2] != 'P')
+	 || !client->request.path_translated) {
 		is_static_file = 1;
 	}
 
 	if (server->router || !is_static_file) {
 		if (FAILURE == php_cli_server_request_startup(server, client)) {
-			SG(server_context) = NULL;
-			php_cli_server_close_connection(server, client);
-			destroy_request_info(&SG(request_info));
+			php_cli_server_request_shutdown(server, client);
 			return SUCCESS;
 		}
 	}
@@ -2177,12 +2239,12 @@ static int php_cli_server_dispatch(php_cli_server *server, php_cli_server_client
 			static int (*send_header_func)(sapi_headers_struct *);
 			send_header_func = sapi_module.send_headers;
 			/* do not generate default content type header */
-		    SG(sapi_headers).send_default_content_type = 0;
+			SG(sapi_headers).send_default_content_type = 0;
 			/* we don't want headers to be sent */
 			sapi_module.send_headers = sapi_cli_server_discard_headers;
 			php_request_shutdown(0);
 			sapi_module.send_headers = send_header_func;
-		    SG(sapi_headers).send_default_content_type = 1;
+			SG(sapi_headers).send_default_content_type = 1;
 			SG(rfc1867_uploaded_files) = NULL;
 		}
 		if (SUCCESS != php_cli_server_begin_send_static(server, client)) {
@@ -2203,6 +2265,7 @@ static int php_cli_server_mime_type_ctor(php_cli_server *server, const php_cli_s
 	const php_cli_server_ext_mime_type_pair *pair;
 
 	zend_hash_init(&server->extension_mime_types, 0, NULL, NULL, 1);
+	GC_MAKE_PERSISTENT_LOCAL(&server->extension_mime_types);
 
 	for (pair = mime_type_map; pair->ext; pair++) {
 		size_t ext_len = strlen(pair->ext);
@@ -2228,18 +2291,144 @@ static void php_cli_server_dtor(php_cli_server *server) /* {{{ */
 	if (server->router) {
 		pefree(server->router, 1);
 	}
+#if HAVE_FORK
+	if (php_cli_server_workers_max > 1 &&
+		php_cli_server_workers &&
+		getpid() == php_cli_server_master) {
+		zend_long php_cli_server_worker;
+
+		for (php_cli_server_worker = 0;
+			 php_cli_server_worker < php_cli_server_workers_max;
+			 php_cli_server_worker++) {
+			 int php_cli_server_worker_status;
+
+			 do {
+				if (waitpid(php_cli_server_workers[php_cli_server_worker],
+						   &php_cli_server_worker_status,
+						   0) == FAILURE) {
+					/* an extremely bad thing happened */
+					break;
+				}
+
+			 } while (!WIFEXITED(php_cli_server_worker_status) &&
+					  !WIFSIGNALED(php_cli_server_worker_status));
+		}
+
+		pefree(php_cli_server_workers, 1);
+	}
+#endif
 } /* }}} */
 
 static void php_cli_server_client_dtor_wrapper(zval *zv) /* {{{ */
 {
 	php_cli_server_client *p = Z_PTR_P(zv);
 
+	shutdown(p->sock, SHUT_RDWR);
 	closesocket(p->sock);
 	php_cli_server_poller_remove(&p->server->poller, POLLIN | POLLOUT, p->sock);
 	php_cli_server_client_dtor(p);
 	pefree(p, 1);
 } /* }}} */
 
+/**
+ * Parse the host and port portions of an address specifier in
+ * one of the following forms:
+ * - hostOrIP:port
+ * - [hostOrIP]:port
+ */
+static char *php_cli_server_parse_addr(const char *addr, int *pport) {
+	const char *p, *end;
+	long port;
+
+	if (addr[0] == '[') {
+		/* Encapsulated [hostOrIP]:port */
+		const char *start = addr + 1;
+		end = strchr(start, ']');
+		if (!end) {
+			/* No ending ] delimiter to match [ */
+			return NULL;
+		}
+
+		p = end + 1;
+		if (*p != ':') {
+			/* Invalid char following address/missing port */
+			return NULL;
+		}
+
+		port = strtol(p + 1, (char**)&p, 10);
+		if (p && *p) {
+			/* Non-numeric in port */
+			return NULL;
+		}
+		if (port < 0 || port > 65535) {
+			/* Invalid port */
+			return NULL;
+		}
+
+		/* Full [hostOrIP]:port provided */
+		*pport = (int)port;
+		return pestrndup(start, end - start, 1);
+	}
+
+	end = strchr(addr, ':');
+	if (!end) {
+		/* Missing port */
+		return NULL;
+	}
+
+	port = strtol(end + 1, (char**)&p, 10);
+	if (p && *p) {
+		/* Non-numeric port */
+		return NULL;
+	}
+	if (port < 0 || port > 65535) {
+		/* Invalid port */
+		return NULL;
+	}
+	*pport = (int)port;
+	return pestrndup(addr, end - addr, 1);
+}
+
+static void php_cli_server_startup_workers(void) {
+	char *workers = getenv("PHP_CLI_SERVER_WORKERS");
+	if (!workers) {
+		return;
+	}
+
+#if HAVE_FORK
+	php_cli_server_workers_max = ZEND_ATOL(workers);
+	if (php_cli_server_workers_max > 1) {
+		zend_long php_cli_server_worker;
+
+		php_cli_server_workers = pecalloc(
+			php_cli_server_workers_max, sizeof(pid_t), 1);
+
+		php_cli_server_master = getpid();
+
+		for (php_cli_server_worker = 0;
+			 php_cli_server_worker < php_cli_server_workers_max;
+			 php_cli_server_worker++) {
+			pid_t pid = fork();
+
+			if (pid == FAILURE) {
+				/* no more forks allowed, work with what we have ... */
+				php_cli_server_workers_max =
+					php_cli_server_worker + 1;
+				return;
+			} else if (pid == SUCCESS) {
+				return;
+			} else {
+				php_cli_server_workers[php_cli_server_worker] = pid;
+			}
+		}
+	} else {
+		fprintf(stderr, "number of workers must be larger than 1\n");
+	}
+#else
+	fprintf(stderr, "forking is not supported on this platform\n");
+#endif
+}
+
 static int php_cli_server_ctor(php_cli_server *server, const char *addr, const char *document_root, const char *router) /* {{{ */
 {
 	int retval = SUCCESS;
@@ -2250,40 +2439,9 @@ static int php_cli_server_ctor(php_cli_server *server, const char *addr, const c
 	int err = 0;
 	int port = 3000;
 	php_socket_t server_sock = SOCK_ERR;
-	char *p = NULL;
 
-	if (addr[0] == '[') {
-		host = pestrdup(addr + 1, 1);
-		if (!host) {
-			return FAILURE;
-		}
-		p = strchr(host, ']');
-		if (p) {
-			*p++ = '\0';
-			if (*p == ':') {
-				port = strtol(p + 1, &p, 10);
-				if (port <= 0 || port > 65535) {
-					p = NULL;
-				}
-			} else if (*p != '\0') {
-				p = NULL;
-			}
-		}
-	} else {
-		host = pestrdup(addr, 1);
-		if (!host) {
-			return FAILURE;
-		}
-		p = strchr(host, ':');
-		if (p) {
-			*p++ = '\0';
-			port = strtol(p, &p, 10);
-			if (port <= 0 || port > 65535) {
-				p = NULL;
-			}
-		}
-	}
-	if (!p) {
+	host = php_cli_server_parse_addr(addr, &port);
+	if (!host) {
 		fprintf(stderr, "Invalid address: %s\n", addr);
 		retval = FAILURE;
 		goto out;
@@ -2291,15 +2449,25 @@ static int php_cli_server_ctor(php_cli_server *server, const char *addr, const c
 
 	server_sock = php_network_listen_socket(host, &port, SOCK_STREAM, &server->address_family, &server->socklen, &errstr);
 	if (server_sock == SOCK_ERR) {
-		php_cli_server_logf("Failed to listen on %s:%d (reason: %s)", host, port, errstr ? ZSTR_VAL(errstr) : "?");
+		php_cli_server_logf(PHP_CLI_SERVER_LOG_ERROR, "Failed to listen on %s:%d (reason: %s)", host, port, errstr ? ZSTR_VAL(errstr) : "?");
 		if (errstr) {
 			zend_string_release_ex(errstr, 0);
 		}
 		retval = FAILURE;
 		goto out;
 	}
+	// server_sock needs to be non-blocking when using multiple processes. Without it, the first process would
+	// successfully accept the connection but the others would block, causing client sockets of the same select
+	// call not to be handled.
+	if (SUCCESS != php_set_sock_blocking(server_sock, 0)) {
+		php_cli_server_logf(PHP_CLI_SERVER_LOG_ERROR, "Failed to make server socket non-blocking");
+		retval = FAILURE;
+		goto out;
+	}
 	server->server_sock = server_sock;
 
+	php_cli_server_startup_workers();
+
 	err = php_cli_server_poller_ctor(&server->poller);
 	if (SUCCESS != err) {
 		goto out;
@@ -2366,14 +2534,15 @@ static int php_cli_server_recv_event_read_request(php_cli_server *server, php_cl
 	char *errstr = NULL;
 	int status = php_cli_server_client_read_request(client, &errstr);
 	if (status < 0) {
-		if (strcmp(errstr, php_cli_server_request_error_unexpected_eof) == 0 && client->parser.state == s_start_req) {
-#if PHP_DEBUG
-			php_cli_server_logf("%s Closed without sending a request; it was probably just an unused speculative preconnection", client->addr_str);
-#endif
-		} else {
-			php_cli_server_logf("%s Invalid request (%s)", client->addr_str, errstr);
+		if (errstr) {
+			if (strcmp(errstr, php_cli_server_request_error_unexpected_eof) == 0 && client->parser.state == s_start_req) {
+				php_cli_server_logf(PHP_CLI_SERVER_LOG_MESSAGE,
+					"%s Closed without sending a request; it was probably just an unused speculative preconnection", client->addr_str);
+			} else {
+				php_cli_server_logf(PHP_CLI_SERVER_LOG_ERROR, "%s Invalid request (%s)", client->addr_str, errstr);
+			}
+			efree(errstr);
 		}
-		efree(errstr);
 		php_cli_server_close_connection(server, client);
 		return FAILURE;
 	} else if (status == 1 && client->request.request_method == PHP_HTTP_NOT_IMPLEMENTED) {
@@ -2435,10 +2604,13 @@ static int php_cli_server_do_event_for_each_fd_callback(void *_params, php_socke
 		struct sockaddr *sa = pemalloc(server->socklen, 1);
 		client_sock = accept(server->server_sock, sa, &socklen);
 		if (!ZEND_VALID_SOCKET(client_sock)) {
-			char *errstr;
-			errstr = php_socket_strerror(php_socket_errno(), NULL, 0);
-			php_cli_server_logf("Failed to accept a client (reason: %s)", errstr);
-			efree(errstr);
+			int err = php_socket_errno();
+			if (err != SOCK_EAGAIN && php_cli_server_log_level >= PHP_CLI_SERVER_LOG_ERROR) {
+				char *errstr = php_socket_strerror(php_socket_errno(), NULL, 0);
+				php_cli_server_logf(PHP_CLI_SERVER_LOG_ERROR,
+					"Failed to accept a client (reason: %s)", errstr);
+				efree(errstr);
+			}
 			pefree(sa, 1);
 			return SUCCESS;
 		}
@@ -2449,16 +2621,17 @@ static int php_cli_server_do_event_for_each_fd_callback(void *_params, php_socke
 		}
 		client = pemalloc(sizeof(php_cli_server_client), 1);
 		if (FAILURE == php_cli_server_client_ctor(client, server, client_sock, sa, socklen)) {
-			php_cli_server_logf("Failed to create a new request object");
+			php_cli_server_logf(PHP_CLI_SERVER_LOG_ERROR, "Failed to create a new request object");
 			pefree(sa, 1);
 			closesocket(client_sock);
 			return SUCCESS;
 		}
-#if PHP_DEBUG
-		php_cli_server_logf("%s Accepted", client->addr_str);
-#endif
+
+		php_cli_server_logf(PHP_CLI_SERVER_LOG_MESSAGE, "%s Accepted", client->addr_str);
+
 		zend_hash_index_update_ptr(&server->clients, client_sock, client);
-		php_cli_server_recv_event_read_request(server, client);
+
+		php_cli_server_poller_add(&server->poller, POLLIN, client->sock);
 	} else {
 		php_cli_server_client *client;
 		if (NULL != (client = zend_hash_index_find_ptr(&server->clients, fd))) {
@@ -2499,9 +2672,11 @@ static int php_cli_server_do_event_loop(php_cli_server *server) /* {{{ */
 		} else {
 			int err = php_socket_errno();
 			if (err != SOCK_EINTR) {
-				char *errstr = php_socket_strerror(err, NULL, 0);
-				php_cli_server_logf("%s", errstr);
-				efree(errstr);
+				if (php_cli_server_log_level >= PHP_CLI_SERVER_LOG_ERROR) {
+					char *errstr = php_socket_strerror(err, NULL, 0);
+					php_cli_server_logf(PHP_CLI_SERVER_LOG_ERROR, "%s", errstr);
+					efree(errstr);
+				}
 				retval = FAILURE;
 				goto out;
 			}
@@ -2558,6 +2733,11 @@ int do_cli_server(int argc, char **argv) /* {{{ */
 				document_root = document_root_tmp;
 #endif
 				break;
+			case 'q':
+				if (php_cli_server_log_level > 1) {
+					php_cli_server_log_level--;
+				}
+				break;
 		}
 	}
 
@@ -2596,33 +2776,25 @@ int do_cli_server(int argc, char **argv) /* {{{ */
 	sapi_module.phpinfo_as_text = 0;
 
 	{
-		char buf[52];
-
-		if (php_cli_server_get_system_time(buf) != 0) {
-			memmove(buf, "unknown time, can't be fetched", sizeof("unknown time, can't be fetched"));
-		}
-
-		printf("PHP %s Development Server started at %s"
-				"Listening on http://%s\n"
-				"Document root is %s\n"
-				"Press Ctrl-C to quit.\n",
-				PHP_VERSION, buf, server_bind_address, document_root);
+		bool ipv6 = strchr(server.host, ':');
+		php_cli_server_logf(
+			PHP_CLI_SERVER_LOG_PROCESS,
+			"PHP %s Development Server (http://%s%s%s:%d) started",
+			PHP_VERSION, ipv6 ? "[" : "", server.host,
+			ipv6 ? "]" : "", server.port);
 	}
 
-#if defined(HAVE_SIGNAL_H) && defined(SIGINT)
+#if defined(SIGINT)
 	signal(SIGINT, php_cli_server_sigint_handler);
-	zend_signal_init();
 #endif
+
+#if defined(SIGPIPE)
+	signal(SIGPIPE, SIG_IGN);
+#endif
+
+	zend_signal_init();
+
 	php_cli_server_do_event_loop(&server);
 	php_cli_server_dtor(&server);
 	return 0;
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
